// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 2010 - Sylvan Brocard
// Copyright (C) 2011 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

/* -------------------------------------------------------------------------- */
/* This gateway uses the new scilab api
/* -------------------------------------------------------------------------- */

// Headers from Scilab:
#include "machine.h"
#include "Scierror.h"
#include "stack-c.h"
#include "api_scilab.h"
#include "localization.h"

// Headers from the gateway:
#include "gw_linalg.h"
#include "gw_linalg_support.h"

/* -------------------------------------------------------------------------- */

/* Usage: C=zhbev(AB) AB must be a complex hermitian band matrix A converted in diagonal format,
using the convention :
--------------------------------
for j from 1 to size(A)
m = min(size(A),j+nsubdiag)
for i from j to m
AB(1+i+j,j) = A(i,j)
-------------------------------
where nsubdiag is the number of non-null subdiagonals of A.
C is the list of eigenvalues of A, sorted in ascending order. */
/*    Ex:  AB = [1 2 3;4 5 0; 6 0 0]; zhbev(AB)
ans =
- 4.094602   
- 2.0337914  
12.128393  */ 


int sci_linalg_zhbev(char* fname)
{ 
	SciErr sciErr;
	int ldab,n,kd;
	int mAB,nAB;
	int one=1;
	int AB = 1;
	int W = 1, Z = 2, WORK = 3, RWORK = 4;
	int *piAddr = NULL;
	int iComplex = 0, iType =0;
	int LWORKMIN, LWORK;
	int INFO;
	double* lW = NULL;
	double* lRWORK = NULL;
	double* lrAB = NULL;
	double* liAB = NULL;
	double* lrZ = NULL;
	double* liZ = NULL;
	double* lrWORK = NULL;
	double* liWORK = NULL;
	doublecomplex* lAB = NULL;
	doublecomplex* lZ = NULL;
	doublecomplex* lWORK = NULL;

	int minlhs=1, minrhs=1, maxlhs=1, maxrhs=1;

	CheckRhs(minrhs,maxrhs) ;
	CheckLhs(minlhs,maxlhs) ;


	//get variable address of the first input argument
	sciErr = getVarAddressFromPosition(pvApiCtx, AB, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}


	//check type and complexity
	sciErr = getVarType(pvApiCtx, piAddr, &iType);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
	if ( iType != sci_matrix ) {
		Scierror(999,_("%s: Wrong type for input argument #%d. Matrix expected.\n"),fname,1);
		return 0;
	}
	iComplex	= isVarComplex(pvApiCtx, piAddr);
	if ( iComplex == 0 ) {
		Scierror(999,_("%s: Wrong content for input argument #%d. Complex matrix expected.\n"),fname,1);
		return 0;
	}

	//get size and data from Scilab memory
	sciErr = getComplexMatrixOfDouble(pvApiCtx, piAddr, &mAB, &nAB, &lrAB, &liAB);      /*      AB     */
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}

	ldab=mAB;  
	n=nAB;
	kd=ldab-1;

	lAB = oGetDoubleComplexFromPointer(lrAB,liAB,ldab * n);


	//Create W
	sciErr = allocMatrixOfDouble(pvApiCtx,Rhs+W, n, one, &lW);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}


	//Create Z
	sciErr = allocComplexMatrixOfDouble(pvApiCtx,Rhs+Z, one, one, &lrZ, &liZ);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
	lZ = oGetDoubleComplexFromPointer(lrZ,liZ,one * one);


	//Create WORK
	LWORK=Maxvol(Rhs+WORK, "z"); /* max memory currently available */
	if (LWORK < n) {
		Scierror(999,"%s:  not enough memory (use stacksize)\n",fname); 
		return 0;
	}
	sciErr = allocComplexMatrixOfDouble(pvApiCtx,Rhs+WORK, n, one, &lrWORK, &liWORK);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}
	lWORK = oGetDoubleComplexFromPointer(lrWORK,liWORK,n * one);


	//Create RWORK
	LWORKMIN = Max(1, 3*(n-2));
	LWORK=Maxvol(Rhs+RWORK, "d"); /* max memory currently available */
	if (LWORK < LWORKMIN) Scierror(999,"%s:  not enough memory (use stacksize)\n",fname); 
	sciErr = allocMatrixOfDouble(pvApiCtx,Rhs+RWORK, LWORKMIN, one, &lRWORK);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}


	C2F(zhbev)("N","L",&n ,&kd ,lAB ,&ldab ,lW,lZ ,&one,lWORK ,lRWORK ,&INFO);

	if (INFO != 0) {
		C2F(errorinfo)("zhbev ", &INFO, 5L);
		return 0;
	}

	/*      Return C (W) */
	LhsVar(1) = Rhs+W;

	vFreeDoubleComplexFromPointer(lAB);
	vFreeDoubleComplexFromPointer(lZ);
	vFreeDoubleComplexFromPointer(lWORK);

	return(0);
}
